@@ -15,7 +15,7 @@ from tempfile import NamedTemporaryFile
 
 from .combine import combine_inputs_with_outputs
 from .compare import NotebookDifference, compare, test_round_trip_conversion
-from .config import load_jupytext_config, prepare_notebook_for_save
+from .config import load_jupytext_config, notebook_formats
 from .formats import (
     _BINARY_FORMAT_OPTIONS,
     _VALID_FORMAT_OPTIONS,
@@ -180,7 +180,14 @@ def parse_jupytext_args(args=None):
         "See also the --opt and --set-formats options for other ways "
         "to operate on the Jupytext metadata.",
     )
-    action.add_argument(
+    parser.add_argument(
+        "--use-source-timestamp",
+        help="Set the modification timestamp of the output file(s) equal"
+        "to that of the source file, and keep the source file and "
+        "its timestamp unchanged.",
+        action="store_true",
+    )
+    parser.add_argument(
         "--warn-only",
         "-w",
         action="store_true",
@@ -481,8 +488,6 @@ def jupytext_single_file(nb_file, args, log):
         if ext:
             fmt = {"extension": ext}
     if fmt:
-        if config:
-            config.set_default_format_options(fmt)
         set_format_options(fmt, args.format_options)
     log(
         "[jupytext] Reading {}{}".format(
@@ -493,7 +498,7 @@ def jupytext_single_file(nb_file, args, log):
         )
     )
 
-    notebook = read(nb_file, fmt=fmt)
+    notebook = read(nb_file, fmt=fmt, config=config)
     if "extension" in fmt and "format_name" not in fmt:
         text_representation = notebook.metadata.get("jupytext", {}).get(
             "text_representation", {}
@@ -501,15 +506,8 @@ def jupytext_single_file(nb_file, args, log):
         if text_representation.get("extension") == fmt["extension"]:
             fmt["format_name"] = text_representation["format_name"]
 
-    if config and "formats" not in notebook.metadata.get("jupytext", {}):
-        default_formats = config.default_formats(nb_file)
-        if default_formats:
-            notebook.metadata.setdefault("jupytext", {})["formats"] = default_formats
-
     # Compute actual extension when using script/auto, and update nb_dest if necessary
     dest_fmt = args.output_format
-    if dest_fmt and config:
-        config.set_default_format_options(dest_fmt)
     if dest_fmt and dest_fmt["extension"] == ".auto":
         dest_fmt = check_auto_ext(dest_fmt, notebook.metadata, "--to")
         if not args.output and nb_file != "-":
@@ -583,11 +581,14 @@ def jupytext_single_file(nb_file, args, log):
     # Read paired notebooks, except if the pair is being created
     nb_files = [nb_file, nb_dest]
     if args.sync:
-        set_prefix_and_suffix(fmt, notebook, nb_file)
+        formats = notebook_formats(
+            notebook, config, nb_file, fallback_on_current_fmt=False
+        )
+        set_prefix_and_suffix(fmt, formats, nb_file)
         if args.set_formats is None:
             try:
                 notebook, inputs_nb_file, outputs_nb_file = load_paired_notebook(
-                    notebook, fmt, nb_file, log, args.pre_commit_mode
+                    notebook, fmt, config, formats, nb_file, log, args.pre_commit_mode
                 )
                 nb_files = [inputs_nb_file, outputs_nb_file]
             except NotAPairedNotebook as err:
@@ -603,11 +604,20 @@ def jupytext_single_file(nb_file, args, log):
     # Pipe the notebook into the desired commands
     prefix = None if nb_file == "-" else os.path.splitext(os.path.basename(nb_file))[0]
     for cmd in args.pipe or []:
-        notebook = pipe_notebook(notebook, cmd, args.pipe_fmt, prefix=prefix)
+        notebook = pipe_notebook(
+            notebook, cmd, args.pipe_fmt, prefix=prefix, warn_only=args.warn_only
+        )
 
     # and/or test the desired commands onto the notebook
     for cmd in args.check or []:
-        pipe_notebook(notebook, cmd, args.pipe_fmt, update=False, prefix=prefix)
+        pipe_notebook(
+            notebook,
+            cmd,
+            args.pipe_fmt,
+            update=False,
+            prefix=prefix,
+            warn_only=args.warn_only,
+        )
 
     if (
         args.execute
@@ -669,7 +679,6 @@ def jupytext_single_file(nb_file, args, log):
             ) from err
 
     # III. ### Possible actions ###
-    modified = args.update_metadata or args.pipe or args.execute
     # a. Test round trip conversion
     if args.test or args.test_strict:
         try:
@@ -685,7 +694,7 @@ def jupytext_single_file(nb_file, args, log):
 
             # Round trip from a text file
             else:
-                with open(nb_file) as fp:
+                with open(nb_file, encoding="utf-8") as fp:
                     org_text = fp.read()
 
                 # If the destination is not ipynb, we convert to/back that format
@@ -693,7 +702,7 @@ def jupytext_single_file(nb_file, args, log):
                     dest_text = writes(notebook, fmt=dest_fmt)
                     notebook = reads(dest_text, fmt=dest_fmt)
 
-                text = writes(notebook, fmt=fmt)
+                text = writes(notebook, fmt=fmt, config=config)
 
                 if args.test_strict:
                     compare(text, org_text)
@@ -736,14 +745,14 @@ def jupytext_single_file(nb_file, args, log):
             _, ext = os.path.splitext(path)
             fmt = copy(fmt or {})
             fmt = long_form_one_format(fmt, update={"extension": ext})
-            new_content = writes(notebook, fmt=fmt)
+            new_content = writes(notebook, fmt=fmt, config=config)
             diff = None
             if not new_content.endswith("\n"):
                 new_content += "\n"
             if not os.path.isfile(path):
                 modified = True
             else:
-                with open(path) as fp:
+                with open(path, encoding="utf-8") as fp:
                     current_content = fp.read()
                 modified = new_content != current_content
                 if modified and args.diff:
@@ -778,7 +787,12 @@ def jupytext_single_file(nb_file, args, log):
         # Otherwise, we only update the timestamp of the text file to make sure
         # they remain more recent than the ipynb file, for compatibility with the
         # Jupytext contents manager for Jupyter
-        if not modified and not path.endswith(".ipynb"):
+        if args.use_source_timestamp:
+            log(
+                f"[jupytext] Setting the timestamp of {shlex.quote(path)} equal to that of {shlex.quote(nb_file)}"
+            )
+            os.utime(path, (os.stat(path).st_atime, os.stat(nb_file).st_mtime))
+        elif not modified and not path.endswith(".ipynb"):
             log(f"[jupytext] Updating the timestamp of {shlex.quote(path)}")
             os.utime(path, None)
 
@@ -815,34 +829,39 @@ def jupytext_single_file(nb_file, args, log):
         else:
             action = ""
 
+        formats = notebook.metadata.get("jupytext", {}).get("formats")
+        formats = long_form_multiple_formats(formats)
+        if formats:
+            try:
+                base_path_out, _ = find_base_path_and_format(nb_dest, formats)
+            except InconsistentPath:
+                # Drop 'formats' if the destination is not part of the paired notebooks
+                formats = {}
+                notebook.metadata.get("jupytext", {}).pop("formats")
+
         lazy_write(nb_dest, fmt=dest_fmt, action=action)
 
+        nb_dest_in_pair = formats and any(
+            os.path.exists(alt_path) and os.path.samefile(nb_dest, alt_path)
+            for alt_path, _ in paired_paths(nb_file, fmt, formats)
+        )
+
+        if (
+            nb_dest_in_pair
+            and os.path.isfile(nb_file)
+            and not nb_file.endswith(".ipynb")
+            and os.path.isfile(nb_dest)
+            and nb_dest.endswith(".ipynb")
+        ):
+            # If the destination is an ipynb file and is in the pair, then we
+            # update the original text file timestamp, as required by our Content Manager
+            # Otherwise Jupyter will refuse to open the paired notebook #335
+            # NB: An alternative is --use-source-timestamp
+            lazy_write(nb_file, update_timestamp_only=True)
+
     # c. Synchronize paired notebooks
-    if args.sync:
-        # Also update the original notebook if the notebook was modified
-        if modified:
-            inputs_nb_file = None
-
-        def write_function(path, fmt):
-            lazy_write(
-                path,
-                fmt,
-                update_timestamp_only=(path == inputs_nb_file),
-            )
-
-        formats = prepare_notebook_for_save(notebook, config, nb_file)
-        write_pair(nb_file, formats, write_function)
-        return untracked_files
-
-    if (
-        os.path.isfile(nb_file)
-        and nb_dest.endswith(".ipynb")
-        and not nb_file.endswith(".ipynb")
-        and notebook.metadata.get("jupytext", {}).get("formats") is not None
-    ):
-        # Update the original text file timestamp, as required by our Content Manager
-        # Otherwise Jupyter will refuse to open the paired notebook #335
-        lazy_write(nb_file, update_timestamp_only=True)
+    elif args.sync:
+        write_pair(nb_file, formats, lazy_write)
 
     return untracked_files
 
@@ -918,11 +937,9 @@ def set_format_options(fmt, format_options):
         fmt[key] = value
 
 
-def set_prefix_and_suffix(fmt, notebook, nb_file):
+def set_prefix_and_suffix(fmt, formats, nb_file):
     """Add prefix and suffix information from jupytext.formats if format and path matches"""
-    for alt_fmt in long_form_multiple_formats(
-        notebook.metadata.get("jupytext", {}).get("formats")
-    ):
+    for alt_fmt in long_form_multiple_formats(formats):
         if alt_fmt["extension"] == fmt["extension"] and fmt.get(
             "format_name"
         ) == alt_fmt.get("format_name"):
@@ -942,10 +959,45 @@ class InconsistentVersions(ValueError):
     """An error raised when two paired files in the git index contain inconsistent representations"""
 
 
-def load_paired_notebook(notebook, fmt, nb_file, log, pre_commit_mode):
-    """Update the notebook with the inputs and outputs of the most recent paired files"""
-    formats = notebook.metadata.get("jupytext", {}).get("formats")
+def file_in_git_index(path):
+    if not os.path.isfile(path):
+        return False
+    return system("git", "status", "--porcelain", path).strip().startswith(("M", "A"))
 
+
+def git_timestamp(path):
+    if not os.path.isfile(path):
+        return None
+
+    # Files that are in the git index are considered most recent
+    if file_in_git_index(path):
+        return float("inf")
+
+    # Return the commit timestamp
+    try:
+        git_ts_str = system("git", "log", "-1", "--pretty=%ct", path).strip()
+    except SystemExit as err:
+        if err.code == 128:
+            # git not initialized
+            git_ts_str = ""
+        else:
+            raise
+
+    if git_ts_str:
+        return float(git_ts_str)
+
+    # The file is not in the git index
+    return get_timestamp(path)
+
+
+def get_timestamp(path):
+    if not os.path.isfile(path):
+        return None
+    return os.lstat(path).st_mtime
+
+
+def load_paired_notebook(notebook, fmt, config, formats, nb_file, log, pre_commit_mode):
+    """Update the notebook with the inputs and outputs of the most recent paired files"""
     if not formats:
         raise NotAPairedNotebook(f"{shlex.quote(nb_file)} is not a paired notebook")
 
@@ -953,44 +1005,31 @@ def load_paired_notebook(notebook, fmt, nb_file, log, pre_commit_mode):
     _, fmt_with_prefix_suffix = find_base_path_and_format(nb_file, formats)
     fmt.update(fmt_with_prefix_suffix)
 
-    def file_in_git_index(path):
-        return system("git", "status", "--porcelain", path).startswith(("M", "A"))
-
-    use_git_index_rather_than_timestamp = pre_commit_mode and file_in_git_index(nb_file)
-
-    def get_timestamp(path):
-        if not os.path.isfile(path):
-            return None
-        if use_git_index_rather_than_timestamp:
-            # Files that are in the git index are considered more recent
-            return file_in_git_index(path)
-        return os.lstat(path).st_mtime
-
     def read_one_file(path, fmt):
         if path == nb_file:
             return notebook
 
         log(f"[jupytext] Loading {shlex.quote(path)}")
-        return read(path, fmt=fmt)
+        return read(path, fmt=fmt, config=config)
 
-    if use_git_index_rather_than_timestamp:
+    if pre_commit_mode and file_in_git_index(nb_file):
         # We raise an error if two representations of this notebook in the git index are inconsistent
         nb_files_in_git_index = sorted(
             (
                 (alt_path, alt_fmt)
                 for alt_path, alt_fmt in paired_paths(nb_file, fmt, formats)
-                if get_timestamp(alt_path)
+                if file_in_git_index(alt_path)
             ),
             key=lambda x: 0 if x[1]["extension"] != ".ipynb" else 1,
         )
 
         if len(nb_files_in_git_index) > 1:
             path0, fmt0 = nb_files_in_git_index[0]
-            with open(path0) as fp:
+            with open(path0, encoding="utf-8") as fp:
                 text0 = fp.read()
             for alt_path, alt_fmt in nb_files_in_git_index[1:]:
-                nb = read(alt_path, fmt=alt_fmt)
-                alt_text = writes(nb, fmt=fmt0)
+                nb = read(alt_path, fmt=alt_fmt, config=config)
+                alt_text = writes(nb, fmt=fmt0, config=config)
                 if alt_text != text0:
                     diff = compare(alt_text, text0, alt_path, path0, return_diff=True)
                     raise InconsistentVersions(
@@ -1001,13 +1040,15 @@ def load_paired_notebook(notebook, fmt, nb_file, log, pre_commit_mode):
                         f"    git reset {shlex.quote(path0)} && git checkout -- {shlex.quote(path0)}\n"
                     )
 
-    inputs, outputs = latest_inputs_and_outputs(nb_file, fmt, formats, get_timestamp)
+    inputs, outputs = latest_inputs_and_outputs(
+        nb_file, fmt, formats, git_timestamp if pre_commit_mode else get_timestamp
+    )
     notebook = read_pair(inputs, outputs, read_one_file)
 
     return notebook, inputs.path, outputs.path
 
 
-def exec_command(command, input=None, capture=False):
+def exec_command(command, input=None, capture=False, warn_only=False):
     """Execute the desired command, and pipe the given input into it"""
     assert isinstance(command, list)
     sys.stdout.write("[jupytext] Executing {}\n".format(" ".join(command)))
@@ -1026,17 +1067,22 @@ def exec_command(command, input=None, capture=False):
         sys.stderr.write(err.decode("utf-8"))
 
     if process.returncode:
-        sys.stderr.write(
-            "[jupytext] Error: The command '{}' exited with code {}\n".format(
-                " ".join(command), process.returncode
-            )
+        msg = f"The command '{' '.join(command)}' exited with code {process.returncode}"
+        hint = (
+            "" if warn_only else " (use --warn-only to turn this error into a warning)"
         )
-        raise SystemExit(process.returncode)
+        sys.stderr.write(
+            f"[jupytext] {'Warning' if warn_only else 'Error'}: {msg}{hint}\n"
+        )
+        if not warn_only:
+            raise SystemExit(process.returncode)
 
     return out
 
 
-def pipe_notebook(notebook, command, fmt="py:percent", update=True, prefix=None):
+def pipe_notebook(
+    notebook, command, fmt="py:percent", update=True, prefix=None, warn_only=False
+):
     """Pipe the notebook, in the desired representation, to the given command. Update the notebook
     with the returned content if desired."""
     if command in ["black", "flake8", "autopep8"]:
@@ -1072,7 +1118,9 @@ def pipe_notebook(notebook, command, fmt="py:percent", update=True, prefix=None)
             tmp.close()
 
             exec_command(
-                [cmd if cmd != "{}" else tmp.name for cmd in command], capture=update
+                [cmd if cmd != "{}" else tmp.name for cmd in command],
+                capture=update,
+                warn_only=warn_only,
             )
 
             if not update:
@@ -1082,7 +1130,9 @@ def pipe_notebook(notebook, command, fmt="py:percent", update=True, prefix=None)
         finally:
             os.remove(tmp.name)
     else:
-        cmd_output = exec_command(command, text.encode("utf-8"), capture=update)
+        cmd_output = exec_command(
+            command, text.encode("utf-8"), capture=update, warn_only=warn_only
+        )
 
         if not update:
             return notebook
